'' =================================================================================================
''
''   File....... jm_bam_rgb.spin
''   Purpose.... Simple PASM implementation of BAM (bit angle modulation) for LEDs
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2009 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 29 AUG 2009
''   Updated.... 31 AUG 2009
''
'' =================================================================================================

{{

   Reference

   * http://www.artisticlicence.com/WebSiteMaster/App%20Notes/appnote011.pdf
   
}}


con

  #0, ALL, RED, GREEN, BLUE

  #0, LINEAR
  
  
var

  long  cog

  byte  redlevel                                                ' brightness, 0 to 255
  byte  grnlevel
  byte  blulevel


pub init(rpin, gpin, bpin) | okay

'' Start RGB LED cog

  finalize

  ' setup cog parameters before launching

  TIX_001 := (clkfreq / 1_000_000) << 1                         ' timing for one "tick"

  rmask := |< rpin                                              ' create pin masks
  gmask := |< gpin
  bmask := |< bpin
       
  rpntr := @redlevel                                            ' set level pointers
  gpntr := @grnlevel
  bpntr := @blulevel

  okay := cog := cognew(@bamrgb, 0) + 1                         ' start the rgb cog

  return okay 


pub finalize

'' Stops serial RGB LED driver; frees a cog

  if cog
    cogstop(cog~ - 1)        


pub setrgb(r, g, b)

'' Set the brightness levels of all three colors

  setred(r)
  setgreen(g)
  setblue(b)


pub setcolor(color, level)

'' Set specified color to level; 0 = all

  case color
    0 : setall(level)
    1 : setred(level)
    2 : setgreen(level)
    3 : setblue(level)
    

pub setall(level)

'' Set all colors to same level

  setred(level)
  setgreen(level)
  setblue(level)

  
pub setred(r)

'' Set the red brightness level

  redlevel := 0 #> r <# 255


pub setgreen(g)

'' Set the green brightness level

  grnlevel := 0 #> g <# 255  


pub setblue(b)

'' Set the blue brightness level

  blulevel := 0 #> b <# 255  


pub ezlog(level)

  level := 0 #> level <#255                                     ' limit to byte value
  
  return (level * level) >> 8                                   ' level := (level^2) / 256


pub setcurve(curvenum, idx)

  case curvenum
    0     : return curve0[idx]
    1     : return curve0[idx]                                  ' assign new tables here
    2     : return curve0[idx]
    3     : return curve0[idx]
    other : return idx                                          ' default


dat

curve0                  byte    000, 001, 002, 003, 004, 005, 006, 007
                        byte    008, 009, 010, 011, 012, 013, 014, 015
                        byte    016, 017, 018, 019, 020, 021, 022, 023
                        byte    024, 025, 026, 027, 028, 029, 030, 031
                        byte    032, 033, 034, 035, 036, 037, 038, 039
                        byte    040, 041, 042, 043, 044, 045, 046, 047
                        byte    048, 049, 050, 051, 052, 053, 054, 055
                        byte    056, 057, 058, 059, 060, 061, 062, 063
                        byte    064, 065, 066, 067, 068, 069, 070, 071
                        byte    072, 073, 074, 075, 076, 077, 078, 079
                        byte    080, 081, 082, 083, 084, 085, 086, 087
                        byte    088, 089, 090, 091, 092, 093, 094, 095
                        byte    096, 097, 098, 099, 100, 101, 102, 103
                        byte    104, 105, 106, 107, 108, 109, 110, 111
                        byte    112, 113, 114, 115, 116, 117, 118, 119
                        byte    120, 121, 122, 123, 124, 125, 126, 127
                        byte    128, 129, 130, 131, 132, 133, 134, 135
                        byte    136, 137, 138, 139, 140, 141, 142, 143
                        byte    144, 145, 146, 147, 148, 149, 150, 151
                        byte    152, 153, 154, 155, 156, 157, 158, 159
                        byte    160, 161, 162, 163, 164, 165, 166, 167
                        byte    168, 169, 170, 171, 172, 173, 174, 175
                        byte    176, 177, 178, 179, 180, 181, 182, 183
                        byte    184, 185, 186, 187, 188, 189, 190, 191
                        byte    192, 193, 194, 195, 196, 197, 198, 199
                        byte    200, 201, 202, 203, 204, 205, 206, 207
                        byte    208, 209, 210, 211, 212, 213, 214, 215
                        byte    216, 217, 218, 219, 220, 221, 222, 223
                        byte    224, 225, 226, 227, 228, 229, 230, 231
                        byte    232, 233, 234, 235, 236, 237, 238, 239
                        byte    240, 241, 242, 243, 244, 245, 246, 247
                        byte    248, 249, 250, 251, 252, 253, 254, 255                         
  
 
dat

                        org     0

bamrgb                  or      dira, rmask                     ' make LED pins outputs
                        or      dira, gmask
                        or      dira, bmask

                        min     TIX_001, #121                   ' prevent waitcnt rollover                        

baminit                 mov     bitmask, #%0000_0001            ' start with bit0
                        mov     bitperiod, TIX_001
                        mov     bittimer, bitperiod             ' initialize bit timer
                        add     bittimer, cnt                   ' sync with system counter

                        mov     tmp1, #0                        ' initialize output workspace

getlevels               rdbyte  rlevel, rpntr                   ' read levels from hub
                        rdbyte  glevel, gpntr
                        rdbyte  blevel, bpntr

bamloop                 test    rlevel, bitmask         wc      ' get bit level
                        muxc    tmp1, rmask                     ' write it to workspace
                        test    glevel, bitmask         wc
                        muxc    tmp1, gmask
                        test    blevel, bitmask         wc
                        muxc    tmp1, bmask
                        mov     outa, tmp1                      ' update outputs

                        shl     bitmask, #1                     ' next bit
                        and     bitmask, #$FF           wz
                if_nz   shl     bitperiod, #1                   ' update bit period                         
                if_z    mov     bitmask, #%0000_0001            ' reset if new cycle
                if_z    mov     bitperiod, TIX_001              ' reset if new cycle

                        waitcnt bittimer, bitperiod             ' finish bit, then reload
                        
                if_nz   jmp     #bamloop                        ' test next bit
                        jmp     #getlevels

' --------------------------------------------------------------------------------------------------

TIX_001                 long    0-0                             ' timing for bit0

rmask                   long    0-0                             ' mask for red pin 
gmask                   long    0-0                             ' mask for green pin 
bmask                   long    0-0                             ' mask for blue pin

rpntr                   long    0-0                             ' pointer to red level
gpntr                   long    0-0                             ' pointer to green level
bpntr                   long    0-0                             ' pointer to blue level

rlevel                  res     1                               ' red level, 0 to 255
glevel                  res     1                               ' green level
blevel                  res     1                               ' blue level

bitmask                 res     1                               ' bit under test
bitperiod               res     1                               ' duration of current bit
bittimer                res     1

tmp1                    res     1

                        fit     492
                        

dat

{{

  Copyright (c) 2009 Jon McPhalen (aka Jon Williams)

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}

                